@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: database.c,v 1.8 1997/12/03 11:50:30 gerd Exp $
+** $Id: database.c,v 1.9 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -38,24 +39,25 @@
 #else
 #define _ARG(A) ()
 #endif
- DB new_db _ARG((void));			   /* database.c             */
- Record db_find _ARG((DB db,char *key));	   /* database.c             */
- char * db_new_key _ARG((DB db,char *key));	   /* database.c             */
- char * db_string _ARG((DB db,char *s,int localp));/* database.c             */
- int *db_count _ARG((DB db,int *lp));		   /* database.c             */
- int read_db _ARG((DB db,char *file,int (*fct)_ARG((DB,Record)),int verbose));/* database.c*/
- static Record insert_record _ARG((Record rec,Record ptr,int (*less)_ARG((Record,Record))));/* database.c*/
- static Record rec__sort _ARG((Record rec,int (*less)_ARG((Record,Record))));/* database.c*/
- static int cmp_heap _ARG((Record r1,Record r2));  /* database.c             */
- static void mark_string _ARG((Record rec,char *s));/* database.c            */
- void db_add_record _ARG((DB db,Record rec));	   /* database.c             */
- void db_forall _ARG((DB db,int (*fct)_ARG((DB,Record))));/* database.c      */
- void db_mac_sort _ARG((DB db));		   /* database.c             */
- void db_rewind _ARG((DB db));			   /* database.c             */
- void db_sort _ARG((DB db,int (*less)_ARG((Record,Record))));/* database.c   */
- void delete_record _ARG((DB db,Record rec));	   /* database.c             */
- void free_db _ARG((DB db));			   /* database.c             */
- void print_db _ARG((FILE *file,DB db,char *spec));/* database.c             */
+ DB new_db _ARG((void));
+ Record db_find _ARG((DB db,Uchar *key));
+ Uchar * db_new_key _ARG((DB db,Uchar *key));
+ Uchar * db_string _ARG((DB db,Uchar *s,int localp));
+ int *db_count _ARG((DB db,int *lp));
+ int *db_count_special _ARG((DB db,int *lp));
+ int read_db _ARG((DB db,char *file,int (*fct)_ARG((DB,Record)),int verbose));
+ static Record insert_record _ARG((Record rec,Record ptr,int (*less)_ARG((Record,Record))));
+ static Record rec__sort _ARG((Record rec,int (*less)_ARG((Record,Record))));
+ static int cmp_heap _ARG((Record r1,Record r2));
+ static void mark_string _ARG((Record rec,Uchar *s));
+ void db_add_record _ARG((DB db,Record rec));
+ void db_forall _ARG((DB db,int (*fct)_ARG((DB,Record))));
+ void db_mac_sort _ARG((DB db));
+ void db_rewind _ARG((DB db));
+ void db_sort _ARG((DB db,int (*less)_ARG((Record,Record))));
+ void delete_record _ARG((DB db,Record rec));
+ void free_db _ARG((DB db));
+ void print_db _ARG((FILE *file,DB db,char *spec));
 
 /*****************************************************************************/
 /* External Programs                                                         */
@@ -182,7 +184,7 @@ int read_db(db,file,fct,verbose)		   /*                        */
   { master_record = new_record(0,32); }		   /*                        */
   RecordSource(master_record) = (file==NULL	   /*                        */
 				 ?sym_empty	   /*                        */
-				 :symbol(file));   /*                        */
+				 :symbol((Uchar*)file));/*                   */
  						   /*                        */
   if ( file == NULL ) file = "<stdin>";		   /*                        */
   if ( verbose ) 			   	   /* If desired print an    */
@@ -236,7 +238,7 @@ int read_db(db,file,fct,verbose)		   /*                        */
 **___________________________________________________			     */
 static void mark_string(rec,s)			   /*                        */
   Record rec;					   /*                        */
-  char   *s;					   /*                        */
+  Uchar   *s;					   /*                        */
 { int    d;					   /*                        */
   Record r;					   /*                        */
  						   /*                        */
@@ -267,13 +269,13 @@ static void mark_string(rec,s)			   /*                        */
 	if ( *s ) s++;
 	break;
       default:
-	if (is_allowed(*s))
-	{ char c, *t;
-	  char *mac = s;
+	if ( is_allowed(*s) )
+	{ Uchar c, *t;
+	  Uchar *mac = s;
 	  while ( is_allowed(*s) ) s++;
 	  c = *s;
 	  *s ='\0';
-	  t = new_string(mac);
+	  t = new_Ustring(mac);
 	  *s = c;
 	  mac = sym_add(t,0);
 	  free(t);
@@ -389,7 +391,7 @@ void print_db(file,db,spec)			   /*                        */
 	{					   /*                        */
 	  if ( allp || RecordIsMARKED(rec) )	   /*                        */
 	  {					   /*                        */
-	    fput_record(file,rec,db,"@");	   /*                        */
+	    fput_record(file,rec,db,(Uchar*)"@");  /*                        */
 	  }					   /*                        */
 	}					   /*                        */
 	else if ( rsc_del_q )			   /*                        */
@@ -491,7 +493,7 @@ void db_forall(db,fct)				   /*                        */
 **___________________________________________________			     */
 Record db_find(db,key)				   /*                        */
   DB              db;				   /*                        */
-  char		  *key;				   /*                        */
+  Uchar		  *key;				   /*                        */
 { register Record rec;				   /*                        */
 						   /*                        */
   if ( DBnormal(db) == RecordNULL ) return RecordNULL;/*                     */
@@ -527,9 +529,9 @@ Record db_find(db,key)				   /*                        */
 **	key	Key to find.
 ** Returns:	nothing
 **___________________________________________________			     */
-char * db_new_key(db,key)			   /*                        */
+Uchar * db_new_key(db,key)			   /*                        */
   DB              db;				   /*                        */
-  char		  *key;				   /*                        */
+  Uchar		  *key;				   /*                        */
 { register Record rec;				   /*                        */
 						   /*                        */
   if ( DBnormal(db) == RecordNULL ) return NULL;   /*                        */
@@ -537,7 +539,8 @@ char * db_new_key(db,key)			   /*                        */
   for ( rec = DBnormal(db);			   /*                        */
 	rec != RecordNULL;			   /*                        */
 	rec = NextRecord(rec) )			   /*                        */
-  { if ( RecordOldKey(rec) != NULL &&		   /*                        */
+  {
+    if ( RecordOldKey(rec) != NULL &&		   /*                        */
 	 RecordOldKey(rec) == key )		   /*                        */
     return *RecordHeap(rec); 			   /*                        */
   }						   /*                        */
@@ -545,7 +548,8 @@ char * db_new_key(db,key)			   /*                        */
   for ( rec = PrevRecord(DBnormal(db));		   /*                        */
 	rec != RecordNULL;			   /*                        */
 	rec = PrevRecord(rec) )			   /*                        */
-  { if ( RecordOldKey(rec) != NULL &&		   /*                        */
+  {
+    if ( RecordOldKey(rec) != NULL &&		   /*                        */
 	 RecordOldKey(rec) == key )		   /*                        */
     return *RecordHeap(rec); 			   /*                        */
   }						   /*                        */
@@ -697,7 +701,8 @@ static Record insert_record(rec,ptr,less)	   /*			     */
 static int cmp_heap(r1,r2)		   	   /*                        */
   register Record r1;				   /*                        */
   register Record r2;				   /*                        */
-{ return strcmp(*RecordHeap(r1),*RecordHeap(r2))<0;/*                        */
+{ return strcmp((char*)*RecordHeap(r1),		   /*                        */
+		(char*)*RecordHeap(r2))<0;	   /*                        */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
@@ -750,9 +755,9 @@ void db_sort(db,less)				   /*                        */
 **	localp	Boolean determining whether the search is only local to the db.
 ** Returns:	The macro expansion or |NULL| upon failure.
 **___________________________________________________			     */
-char * db_string(db,s,localp)			   /*                        */
+Uchar * db_string(db,s,localp)			   /*                        */
   DB     db;					   /*                        */
-  char   *s;					   /*                        */
+  Uchar   *s;					   /*                        */
   int    localp;				   /*                        */
 { Record rec;					   /*                        */
  						   /*                        */
@@ -792,6 +797,45 @@ char * db_string(db,s,localp)			   /*                        */
 ** Returns:	Static array containing the statistics.
 **___________________________________________________			     */
 int *db_count(db,lp)				   /*                        */
+  DB     db;				   	   /*                        */
+  int    *lp;				   	   /*                        */
+{ int    *count;				   /*                        */
+  Record rec;				   	   /*                        */
+ 						   /*                        */
+  count = db_count_special(db,lp);		   /*                        */
+					       	   /*                        */
+  if ( (rec=DBnormal(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[RecordType(rec)]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+						   /*                        */
+  return count;					   /*                        */
+}						   /*------------------------*/
+
+/*-----------------------------------------------------------------------------
+** Function:	db_count_special()
+** Purpose:	Count all special entries in a database. This includes normal
+**		aswell as special records. The result is stored in a
+**		static array which is reused by |db_count()|. A
+**		pointer to this array is returned. The indices
+**		correspond to the entry types defined with
+**		|add_entry_type()| or declared as symbolic constants
+**		in |entry.h|. 
+**
+**		The end of the array is marked by an element
+**		containing a negative number. In addition the argument
+**		|lp| can point to an integer where the number of valid
+**		elements is stored. If |lp| is |NULL| this step is
+**		omitted. 
+** Arguments:
+**	db	Database to count.
+**	lp	pointer to an integer for the length.
+** Returns:	Static array containing the statistics.
+**___________________________________________________			     */
+int *db_count_special(db,lp)			   /*                        */
   DB         db;				   /*                        */
   int        *lp;				   /*                        */
 { static int *count;				   /*                        */
@@ -817,55 +861,48 @@ int *db_count(db,lp)				   /*                        */
   for ( i=0; i<len; i++ ) count[i] = 0;		   /*                        */
   count[len-1] = -1;				   /*                        */
 						   /*                        */
-  if ( (rec=DBstring(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_STRING]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBcomment(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_COMMENT]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBpreamble(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_PREAMBLE]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBalias(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_ALIAS]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBmodify(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_MODIFY]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBinclude(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[BIB_INCLUDE]++;
-      rec = NextRecord(rec);
-    }
-  }
-  if ( (rec=DBnormal(db)) != RecordNULL )
-  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);
-    while ( rec != RecordNULL )
-    { count[RecordType(rec)]++;
-      rec = NextRecord(rec);
-    }
-  }
-						   /*                        */
+  if ( (rec=DBstring(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_STRING]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+  if ( (rec=DBcomment(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_COMMENT]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+  if ( (rec=DBpreamble(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_PREAMBLE]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+  if ( (rec=DBalias(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_ALIAS]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+  if ( (rec=DBmodify(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_MODIFY]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+  if ( (rec=DBinclude(db)) != RecordNULL )	   /*                        */
+  { while ( PrevRecord(rec) != RecordNULL ) rec = PrevRecord(rec);/*         */
+    while ( rec != RecordNULL )			   /*                        */
+    { count[BIB_INCLUDE]++;			   /*                        */
+      rec = NextRecord(rec);			   /*                        */
+    }						   /*                        */
+  }						   /*                        */
+ 						   /*                        */
   return count;					   /*                        */
 }						   /*------------------------*/
