@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: error.c,v 2.17 1997/12/03 11:50:30 gerd Exp $
+** $Id: error.c,v 2.18 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -38,16 +39,34 @@
 
 /*---------------------------------------------------------------------------*/
 
+ char  *err_fmt1 = "%s BibTool: %s\n";
+ char  *err_fmt2 = "%s BibTool: %s%s\n";
+ char  *err_fmt3 = "%s BibTool: %s%s%s\n";
+ char  *err_fmt4 = "%s BibTool: %s%s%s%s\n";
  char  *err_format = "*** BibTool: %s";
- Uchar *err_point  = ".";
- Uchar *err_oom    = "Out of memory for ";
+ Uchar *err_point  = (Uchar*)".";
+ Uchar *err_oom    = (Uchar*)"Out of memory for ";
 
- FILE * err_file   = stderr;
+ FILE * err_file   = NULL;
 
 #define ErrNL		(void)fputc('\n',err_file)
 #define ErrChar(C)	(void)fputc(C,err_file)
 #define ErrS(S)		(void)fputs(S,err_file)
 
+/*-----------------------------------------------------------------------------
+** Function:	init_error()
+** Type:	void
+** Purpose:	
+**		
+**		
+** Arguments:
+**		
+** Returns:	nothing
+**___________________________________________________			     */
+void init_error()				   /*                        */
+{ err_file = stderr;				   /*                        */
+}						   /*------------------------*/
+
 /*-----------------------------------------------------------------------------
 ** Function:	error()
 ** Purpose:	This is the generic error printing routine.
