@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: expand.h,v 1.10 1997/12/03 11:50:30 gerd Exp $
+** $Id: expand.h,v 1.11 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -23,4 +24,4 @@
 #else
 #define _ARG(A) ()
 #endif
- char * expand_rhs _ARG((char *s,char *pre,char *post, DB db));/* expand.c   */
+ Uchar * expand_rhs _ARG((Uchar *s,char *pre,char *post, DB db));/* expand.c */
