@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: print.h,v 2.18 1997/12/03 11:50:30 gerd Exp $
+** $Id: print.h,v 2.19 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -28,8 +29,8 @@
 #else
 #define _ARG(A) ()
 #endif
- char * sput_record _ARG((Record rec,DB db,char *start));/* print.c          */
- void fput_record _ARG((FILE *file,Record rec,DB db,char *start));/* print.c */
- void put_record _ARG((int (*fct)_ARG((int)),Record rec,DB db,char *start));/* print.c*/
+ char * sput_record _ARG((Record rec,DB db,Uchar *start));/* print.c         */
+ void fput_record _ARG((FILE *file,Record rec,DB db,Uchar *start));/* print.c*/
+ void put_record _ARG((int (*fct)_ARG((int)),Record rec,DB db,Uchar *start));/* print.c*/
  void set_key_type _ARG((char * s));		   /* print.c                */
  void set_symbol_type _ARG((char * s));		   /* print.c                */
