@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: stack.h,v 2.12 1997/08/25 06:32:05 gerd Exp $
+** $Id: stack.h,v 2.14 1999/12/19 08:52:25 gerd Exp gerd $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -18,10 +19,12 @@
 ** 
 ******************************************************************************/
 
+#include <bibtool/type.h>
+
 #ifdef __STDC__
 #define _ARG(A) A
 #else
 #define _ARG(A) ()
 #endif
- char * pop_string _ARG((void));		   /* stack.c                */
- void push_string _ARG((char * s));		   /* stack.c                */
+ Uchar * pop_string _ARG((void));		   /* stack.c                */
+ void push_string _ARG((Uchar * s));		   /* stack.c                */
