@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: main.c,v 2.34 1997/12/06 22:56:00 gerd Exp gerd $
+** $Id: main.c,v 2.35 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 ** >>> See the file Doc/c_lib.tex for a printed documentation of the
 ** >>> main parts of the C source.
@@ -64,6 +65,7 @@
 #include <bibtool/symbols.h>
 #include <bibtool/sbuffer.h>
 #include <bibtool/expand.h>
+#include <bibtool/version.h>
 #ifdef HAVE_LIBKPATHSEA
 #include <kpathsea/debug.h>
 #endif
@@ -78,17 +80,19 @@
 #else
 #define _ARG(A) ()
 #endif
- int main _ARG((int argc,char *argv[]));	   /* main.c                 */
- static int dbl_check _ARG((DB db,Record rec));	   /* main.c                 */
- static int do_keys _ARG((DB db,Record rec));	   /* main.c                 */
- static int do_no_keys _ARG((DB db,Record rec));   /* main.c                 */
- static int rec_gt _ARG((Record r1,Record r2));	   /* main.c                 */
- static int rec_lt _ARG((Record r1,Record r2));	   /* main.c                 */
- static int update_crossref _ARG((DB db,Record rec));/* main.c               */
- static void usage _ARG((int fullp));		   /* main.c                 */
- void save_input_file _ARG((char *file));	   /* main.c                 */
- void save_macro_file _ARG((char *file));	   /* main.c                 */
- void save_output_file _ARG((char * file));	   /* main.c                 */
+ int main _ARG((int argc,char *argv[]));
+ static int dbl_check _ARG((DB db,Record rec));
+ static int do_keys _ARG((DB db,Record rec));
+ static int do_no_keys _ARG((DB db,Record rec));
+ static int rec_gt _ARG((Record r1,Record r2));
+ static int rec_lt _ARG((Record r1,Record r2));
+ static int rec_read _ARG((DB db,Record rec));
+ static int update_crossref _ARG((DB db,Record rec));
+ static void usage _ARG((int fullp));
+ void printchar _ARG((char c));
+ void save_input_file _ARG((char *file));
+ void save_macro_file _ARG((char *file));
+ void save_output_file _ARG((char * file));
 
 /*****************************************************************************/
 /* External Programs and Variables					     */
@@ -149,8 +153,8 @@ char * getenv(name)				   /*			     */
     "\t%c$\t\tSymbol table output (debugging only)\n",
 #endif
     0L,
-    "Copyright (C) Gerd Neugebauer $Date: 1997/12/06 22:56:00 $",
-    "gerd@informatik.uni-koblenz.de"
+    "Copyright (C) Gerd Neugebauer $Date: 1999/12/18 22:04:46 $",
+    "gerd.neugebauer@sdm.de"
   };
 
 /*-----------------------------------------------------------------------------
@@ -328,6 +332,10 @@ void save_macro_file(file)			   /*			     */
 
 #define Toggle(X) X = !(X)
 
+ static int stat_len;				   /*                        */
+ static int *stat_count;			   /*                        */
+
+
 /*-----------------------------------------------------------------------------
 ** Function:	main()
 ** Purpose:	This is the main function which is automatically
@@ -357,9 +365,6 @@ int main(argc,argv)				   /*			     */
   FILE	*file;				   	   /*                        */
   int	need_rsc = TRUE;		   	   /*			     */
   int	(*fct)();			   	   /* Function pointer	     */
-  int	c_len;					   /*                        */
-  int   *c;					   /*                        */
- 						   /*                        */
   init_bibtool(argv[0]);			   /*                        */
 						   /*			     */
   for ( i=1; i<argc; i++ )			   /*			     */
@@ -368,10 +373,10 @@ int main(argc,argv)				   /*			     */
     }						   /*			     */
     else					   /*			     */
     { switch ( *++ap )				   /*			     */
-      { case 'A': set_base(ap+1);	    break; /* disambiguation	     */
-	case 'd': Toggle(rsc_double_check); break; /* double entries	     */
+      { case 'A': set_base((Uchar*)(ap+1));  break;/* disambiguation	     */
+	case 'd': Toggle(rsc_double_check);  break;/* double entries	     */
 #ifdef HAVE_LIBKPATHSEA
-	case 'D': kpathsea_debug = atoi(++ap); break;/* kpathsea debugging   */
+	case 'D': kpathsea_debug=atoi(++ap); break;/* kpathsea debugging     */
 #endif
 	case 'f': add_format(argv[++i]);	   /*	!!! no break !!!     */
 	case 'F': rsc_make_key = TRUE;	    break; /* key generation	     */
@@ -411,14 +416,14 @@ int main(argc,argv)				   /*			     */
 	  break;				   /*			     */
 	case 'X':				   /* extract pattern	     */
 	  rsc_all_macs = FALSE;			   /*                        */
-	  if ( ++i < argc ) { save_regex(argv[i]); }/*			     */
+	  if ( ++i < argc ) { save_regex((Uchar*)argv[i]); }/*		     */
 	  else		    { MissingPattern; }	   /*			     */
 	  break;				   /*			     */
 	case '#': Toggle(rsc_cnt_all);	    break; /* print full statistics  */
 	case '@': Toggle(rsc_cnt_used);	    break; /* print short statistics */
 	case '-':				   /* extended command	     */
-	  if ( *++ap )	      { (void)use_rsc(ap); }/*			     */
-	  else if ( ++i<argc ){ (void)use_rsc(argv[i]); }/*		     */
+	  if ( *++ap )	      { (void)use_rsc((Uchar*)ap); }/*		     */
+	  else if ( ++i<argc ){ (void)use_rsc((Uchar*)argv[i]); }/*	     */
 	  else		      { MissingResource;  }/*			     */
 	  break;				   /*			     */
 #ifdef SYMBOL_DUMP
@@ -440,35 +445,45 @@ int main(argc,argv)				   /*			     */
   init_read();					   /* Just in case the path  */
  						   /*  has been modified.    */
   the_db = new_db();				   /*                        */
+  						   /*                        */
+  (void) db_count(the_db,&stat_len);	   	   /* Determine the numer of */
+ 						   /* registered record types*/
+  if ( (stat_count=(int*)malloc(stat_len*sizeof(int))) == NULL )/*           */
+  { OUT_OF_MEMORY("count"); }			   /*                        */
+  else					   	   /*                        */
+  { for (i=0;i<stat_len;i++)			   /*                        */
+    { stat_count[i] = 0; }			   /*                        */
+  }						   /*                        */
 						   /*			     */
   for ( i=0; i<input_file_ptr; i++ )		   /* For all input files    */
   {						   /*			     */
     if ( read_db(the_db,			   /*                        */
 		 input_files[i],		   /*                        */
-		 is_selected,			   /*                        */
+		 rec_read,			   /*                        */
 		 rsc_verbose) )	   		   /*                        */
     { NoFileError(input_files[i]); }		   /*			     */
   }						   /*			     */
  						   /*                        */
-  apply_aux(the_db);				   /*                        */
- 						   /*                        */
   if ( rsc_cnt_all || rsc_cnt_used )		   /*			     */
-  { int i;
-    int * cnt = db_count(the_db,&c_len);
-
-    if ( (c=(int*)malloc(c_len*sizeof(int))) == NULL )
-    { rsc_cnt_all = rsc_cnt_used = 0; }
-    else
-    { for (i=0;i<c_len;i++) c[i] = cnt[i];
-    }
-  }
+  { int i;					   /*                        */
+    int *count = db_count_special(the_db,(int*)NULL);/*                      */
+						   /*                        */
+    for (i=0; i<stat_len && IsSpecialRecord(i); ++i)/*			     */
+    { stat_count[i] = count[i];			   /*			     */
+    }						   /*			     */
+  }						   /*                        */
+ 						   /*                        */
+  apply_aux(the_db);				   /*                        */
 						   /*			     */
   if ( rsc_make_key )				   /*                        */
-  { if ( rsc_key_preserve )	   		   /*                        */
+  {						   /*                        */
+    start_key_gen();				   /*                        */
+    if ( rsc_key_preserve )	   		   /*                        */
     { db_forall(the_db,mark_key); }		   /*                        */
     db_rewind(the_db);				   /*                        */
     db_forall(the_db,do_keys);		   	   /*                        */
     db_forall(the_db,update_crossref);	   	   /*                        */
+    end_key_gen();				   /*                        */
   }						   /*                        */
   else						   /*                        */
   {						   /*                        */
@@ -490,7 +505,7 @@ int main(argc,argv)				   /*			     */
   if ( output_file == NULL ||			   /*                        */
        (file=fopen(output_file,"w")) == NULL )	   /*                        */
   { file = stdout; }				   /*                        */
-  print_db(file,the_db,rsc_print_et);		   /*                        */
+  print_db(file,the_db,(char*)rsc_print_et);	   /*                        */
   if ( file != stdout ) { fclose(file); }	   /*                        */
 						   /*			     */
   if ( macro_file != NULL  && *macro_file )	   /*			     */
@@ -506,16 +521,16 @@ int main(argc,argv)				   /*			     */
 						   /*			     */
   if ( rsc_cnt_all || rsc_cnt_used )		   /*			     */
   { int i;					   /*                        */
-    int *cnt = db_count(the_db,NULL);		   /*                        */
+    int *count = db_count(the_db,(int*)NULL);	   /*                        */
 						   /*                        */
     (void)fputc('\n',err_file);		   	   /*			     */
-    for (i=0; i<c_len; ++i)			   /*			     */
-    { if ( rsc_cnt_all || c[i] > 0 )	   	   /*			     */
+    for (i=0; i<stat_len; ++i)			   /*			     */
+    { if ( rsc_cnt_all || stat_count[i] > 0 )	   /*			     */
       { (void)fprintf(err_file,			   /*			     */
 		      "---  %-15s %5d read  %5d written\n",/*		     */
 		      get_entry_type(i),	   /*			     */
-		      c[i],		   	   /*			     */
-		      cnt[i]);		   	   /*			     */
+		      stat_count[i],		   /*			     */
+		      count[i]);		   /*			     */
       }						   /*			     */
     }						   /*			     */
   }						   /*                        */
@@ -535,6 +550,23 @@ int main(argc,argv)				   /*			     */
   return 0;					   /*			     */
 }						   /*------------------------*/
 
+/*-----------------------------------------------------------------------------
+** Function:	rec_read()
+** Type:	static int
+** Purpose:	This is a boolean callback function used whenever a
+**		record is read. 
+** Arguments:
+**	db	The current database
+**	rec	The current record
+** Returns:	|TRUE| iff the record should be inserted into the database.
+**___________________________________________________			     */
+static int rec_read(db,rec)			   /*                        */
+  DB db;					   /*                        */
+  Record rec;					   /*                        */
+{ stat_count[RecordType(rec)]++;		   /*                        */
+  return is_selected(db,rec);			   /*                        */
+}						   /*------------------------*/
+
 /*-----------------------------------------------------------------------------
 ** Function:	rec_gt()
 ** Purpose:	
@@ -548,7 +580,8 @@ int main(argc,argv)				   /*			     */
 static int rec_gt(r1,r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
-{ return (strcmp(RecordSortkey(r1),RecordSortkey(r2)) < 0);/*                */
+{ return (strcmp((char*)RecordSortkey(r1),	   /*                        */
+		 (char*)RecordSortkey(r2)) < 0);   /*                        */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
@@ -564,7 +597,8 @@ static int rec_gt(r1,r2)			   /*                        */
 static int rec_lt(r1,r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
-{ return (strcmp(RecordSortkey(r1),RecordSortkey(r2)) > 0);/*                */
+{ return (strcmp((char*)RecordSortkey(r1),	   /*                        */
+		 (char*)RecordSortkey(r2)) > 0);   /*                        */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
@@ -620,11 +654,11 @@ static int do_no_keys(db,rec)			   /*                        */
 ** Returns:	nothing
 **___________________________________________________			     */
 static int update_crossref(db,rec)		   /*			     */
-  DB		db;				   /*                        */
-  Record	rec;				   /*			     */
-{ register char	**hp;				   /*			     */
-  register int  i;				   /*                        */
-  char		*t,*s;			   	   /*			     */
+  DB		 db;				   /*                        */
+  Record	 rec;				   /*			     */
+{ register Uchar **hp;				   /*			     */
+  register int   i;				   /*                        */
+  Uchar		 *t,*s;			   	   /*			     */
 						   /*			     */
   if ( !RecordIsXREF(rec) ) return 0;		   /*			     */
 						   /*                        */
@@ -639,17 +673,17 @@ static int update_crossref(db,rec)		   /*			     */
 						   /*			     */
   t = *++hp; t++;				   /*			     */
   (void)sp_open(t);				   /* Try to extract	     */
-  if ( (s = SParseSymbol(&t)) == NULL )	   	   /*  the crossref as symbol*/
+  if ( (s = SParseSymbol(&t)) == (Uchar)NULL )	   /*  the crossref as symbol*/
   { return 0; }					   /*			     */
 						   /*			     */
-  if ( (s = db_new_key(db,s)) == NULL )	   	   /*			     */
-  { ERROR2("Crossref not found: ",s);		   /*			     */
+  if ( (s = db_new_key(db,s)) == (Uchar)NULL )	   /*			     */
+  { ERROR2("Crossref not found: ",(char*)s);	   /*			     */
     return 0;					   /*			     */
   }						   /*			     */
-  if ( (t = malloc(strlen(s)+3)) == NULL )	   /* allocate temp memory   */
+  if ( (t=(Uchar*)malloc(strlen((char*)s)+3))==(Uchar*)NULL )/* get temp mem */
   { OUT_OF_MEMORY("update_crossref()"); }	   /*		             */
 						   /*			     */
-  (void)sprintf(t,(**hp=='"'?"\"%s\"":"{%s}"),s);  /* construct new crossref */
+  (void)sprintf((char*)t,(**hp=='"'?"\"%s\"":"{%s}"),s);/* make new crossref */
   *hp = symbol(t);				   /* store new crossref     */
   free(t);					   /* free temp memory	     */
   return 0;					   /*                        */
@@ -673,9 +707,21 @@ static int dbl_check(db,rec)			   /*                        */
 {						   /*                        */
   if ( PrevRecord(rec) != RecordNULL		   /*                        */
        && equal_records(PrevRecord(rec),rec) )	   /*			     */
-  { WARNING2("Possible double entries discovered: \n***\t\t",/*	             */
-	     RecordSortkey(rec));		   /*			     */
-    if (rsc_del_dbl)				   /*                        */
+  {						   /*                        */
+    if ( !rsc_quiet )				   /*                        */
+    { Uchar *k1 = *RecordHeap(rec);		   /*                        */
+      Uchar *k2 = *RecordHeap(PrevRecord(rec));	   /*                        */
+      ErrPrint("*** BibTool WARNING: Possible double entries discovered: \n***\t");
+      if ( k1 == (Uchar*)NULL ) k1 = (Uchar*)"";   /*                        */
+      if ( k2 == (Uchar*)NULL ) k2 = (Uchar*)"";   /*                        */
+      ErrPrint(k2);				   /*                        */
+      ErrPrint(" =?= ");			   /*                        */
+      ErrPrint(k1);				   /*                        */
+      ErrPrint("\n***\t");			   /*                        */
+      ErrPrint(RecordSortkey(rec));		   /*			     */
+      ErrPrint("\n");				   /*                        */
+    }						   /*                        */
+    if ( rsc_del_dbl )				   /*                        */
     { delete_record(db,rec); }		   	   /*                        */
     else 					   /*                        */
     { SetRecordDELETED(rec); }			   /*                        */
@@ -684,7 +730,6 @@ static int dbl_check(db,rec)			   /*                        */
   return 0;					   /*                        */
 }						   /*------------------------*/
 
-
 #ifdef DEBUG
 /*-----------------------------------------------------------------------------
 ** Function*:	printchar()
