@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: rsc.c,v 2.24 1997/12/06 22:56:00 gerd Exp gerd $
+** $Id: rsc.c,v 2.25 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 **-----------------------------------------------------------------------------
 ** Description:
@@ -33,6 +34,7 @@
 #include <bibtool/key.h>
 #include <bibtool/rewrite.h>
 #include <bibtool/s_parse.h>
+#include <bibtool/sbuffer.h>
 #include <bibtool/entry.h>
 #include <bibtool/version.h>
 #include "config.h"
@@ -50,9 +52,9 @@
 #endif
  int load_rsc _ARG((char *name));		   /* rsc.c		     */
  int search_rsc _ARG((void));			   /* rsc.c		     */
- int set_rsc _ARG((char * name,char * val));	   /* rsc.c		     */
- int use_rsc _ARG((char * s));			   /* rsc.c		     */
- static int test_true _ARG((char * s));		   /* rsc.c		     */
+ int set_rsc _ARG((Uchar * name,Uchar * val));	   /* rsc.c		     */
+ int use_rsc _ARG((Uchar * s));			   /* rsc.c		     */
+ static int test_true _ARG((Uchar * s));	   /* rsc.c		     */
  static void init_rsc _ARG((void));		   /* rsc.c		     */
  void rsc_print _ARG((char *s));		   /* rsc.c		     */
 
@@ -72,10 +74,10 @@
 
 /*---------------------------------------------------------------------------*/
 
-#define RscNumeric(SYM,S,V,I) static char *S = NULL;
-#define RscString(SYM,S,V,I)  static char *S = NULL;
-#define RscBoolean(SYM,S,V,I) static char *S = NULL;
-#define RscByFct(SYM,S,FCT)   static char *S = NULL;
+#define RscNumeric(SYM,S,V,I) static Uchar *S = (Uchar*)NULL;
+#define RscString(SYM,S,V,I)  static Uchar *S = (Uchar*)NULL;
+#define RscBoolean(SYM,S,V,I) static Uchar *S = (Uchar*)NULL;
+#define RscByFct(SYM,S,FCT)   static Uchar *S = (Uchar*)NULL;
 #include <bibtool/resource.h>
 
 /*-----------------------------------------------------------------------------
@@ -96,10 +98,10 @@
 **___________________________________________________			     */
 static void init_rsc()				   /*			     */
 {						   /*			     */
-#define RscNumeric(SYM,S,V,I)  S = symbol(SYM);
-#define RscString(SYM,S,V,I)   S = symbol(SYM);
-#define RscBoolean(SYM,S,V,I)  S = symbol(SYM);
-#define RscByFct(SYM,S,FCT)    S = symbol(SYM);
+#define RscNumeric(SYM,S,V,I)  S = symbol((Uchar*)SYM);
+#define RscString(SYM,S,V,I)   S = symbol((Uchar*)SYM);
+#define RscBoolean(SYM,S,V,I)  S = symbol((Uchar*)SYM);
+#define RscByFct(SYM,S,FCT)    S = symbol((Uchar*)SYM);
 #ifndef MAKEDEPEND
 #include <bibtool/resource.h>
 #endif
@@ -123,38 +125,70 @@ static void init_rsc()				   /*			     */
 **		For each step |load_rsc()| is called until it
 **		succeeds.
 **
-**		The files sought is determined by the macro
-**		|DefaultResourceFile| at compile time of the
-**		module. (see |bibtool.h|)
+**		The files sought are determined by the macro
+**		|DefaultResourceFiles| at compile time of the
+**		module. (see |config.h|)
 ** Arguments:	none
 ** Returns:	|TRUE| iff the resource loading succeeds somewhere.
 **___________________________________________________			     */
 int search_rsc()				   /*			     */
-{ static char	*def = DefaultResourceFile;	   /*			     */
-  register char *ap;				   /*			     */
-  register char *fn;				   /*			     */
-  int		l;				   /*			     */
+{ static char  *name[] = { DefaultResourceFiles, NULL};/*		     */
+  int          filestart;			   /*                        */
+  char         **np;				   /*                        */
+  int          i;				   /*                        */
+  char         *path[3];			   /*                        */
+  StringBuffer *sb;				   /*                        */
+  char         *p;				   /*                        */
 						   /*			     */
 #ifdef RSC_ENV_VAR
-  if ( (ap=getenv(RSC_ENV_VAR)) != NULL		   /* Try to get the name    */
-      && load_rsc(ap) ) return TRUE;		   /*  from the environment. */
+  if ( (p=getenv(RSC_ENV_VAR)) != NULL )	   /* Try to get the name    */
+  { char *cp, *s;				   /*  from the environment. */
+    char *sep = ENV_SEP;			   /*                        */
+    int loaded = 0;				   /*                        */
+    p = s = new_string(p);			   /*                        */
+    while ( cp=strchr(s,*sep) )			   /*                        */
+    { *cp = '\0';				   /*                        */
+      loaded |= load_rsc(s);			   /*                        */
+      s = cp+1;					   /*                        */
+    }						   /*                        */
+    loaded |= load_rsc(s);			   /*                        */
+    free(p);					   /*                        */
+    if (loaded) return TRUE;			   /*                        */
+  }						   /*                        */
 #endif
 						   /*			     */
+  sb = sbopen();				   /*                        */
 #ifdef HOME_ENV_VAR
-  if ( (ap=getenv(HOME_ENV_VAR)) != NULL )	   /* Try to get the resource*/
-  { l = strlen(ap)+strlen(DIR_SEP)+strlen(def)+1;  /*  file from the home    */
-    if ( (fn=malloc(l)) != NULL )		   /*  directory	     */
-    { (void)strcpy(fn,ap);			   /*			     */
-      (void)strcat(fn,DIR_SEP);			   /*			     */
-      (void)strcat(fn,def);			   /*			     */
-      l = load_rsc(fn);				   /*			     */
-      free(fn);					   /*			     */
-      if ( l ) return TRUE;			   /*			     */
-    }						   /*			     */
-  }						   /*			     */
+  p = getenv(HOME_ENV_VAR);			   /*                        */
+#else
+  p = NULL;					   /*                        */
 #endif
-						   /* if all fails then try  */
-  return load_rsc(def);				   /*  to use a default.     */
+  i = 0;					   /*                        */
+  if ( p != NULL ) path[i++] = p;		   /*                        */
+  path[i++] = "";				   /*                        */
+  path[i] = NULL;				   /*                        */
+ 						   /*                        */
+  for (i=0; (p=path[i]) != NULL;i++)
+  { if ( p != NULL )
+    { (void)sbrewind(sb);
+      if ( *p )
+      { (void)sbputs(p,sb);			   /*			     */
+	(void)sbputs(DIR_SEP,sb);		   /*			     */
+      }
+      filestart = sbtell(sb);
+
+      for (np=name; *np != NULL; np++)
+      { (void)sbseek(sb,filestart);
+	(void)sbputs(*np,sb);
+	if ( load_rsc(sbflush(sb)) )
+	{ sbclose(sb);
+          return TRUE;
+	}
+      }
+    }
+  }
+  sbclose(sb);
+  return FALSE;
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
@@ -187,9 +221,9 @@ int load_rsc(name)				   /*			     */
 ** Returns:	|TRUE| iff the string represents true.
 **___________________________________________________			     */
 static int test_true(s)				   /*			     */
-  register char * s;				   /*			     */
+  Uchar * s;				   	   /*			     */
 {						   /*			     */
-  switch ( *s )
+  switch ( *s )					   /*                        */
   { case '1':		return (s[1] == '\0');
     case 'o': case 'O':
       return ((s[1] == 'n' || s[1] == 'N') &&
@@ -221,9 +255,9 @@ static int test_true(s)				   /*			     */
 ** Returns:	|FALSE| iff no error has occurred.
 **___________________________________________________			     */
 int use_rsc(s)					   /*			     */
-  char		*s;				   /*			     */
-{ register char *name,				   /*			     */
-		*value;				   /*			     */
+  Uchar		*s;				   /*			     */
+{ register Uchar *name,				   /*			     */
+		 *value;			   /*			     */
 						   /*			     */
   (void)sp_open(s);				   /*			     */
   if ( (name = SParseSymbol(&s)) == NULL ) return 1; /*			     */
@@ -251,15 +285,19 @@ int use_rsc(s)					   /*			     */
 ** Returns:	|FALSE| iff everything went right.
 **___________________________________________________			     */
 int set_rsc(name,val)				   /*			     */
-  register char * name;				   /*			     */
-  register char * val;				   /*			     */
+  Uchar *name;				   	   /*			     */
+  Uchar *val;				   	   /*			     */
 {						   /*			     */
   if ( rsc_verbose )				   /*			     */
-    VerbosePrint4("Resource ",name," = ",(val==NULL?"*NULL*":val));/*	     */
+  { VerbosePrint4("Resource ",			   /*                        */
+		  (char*)name,			   /*                        */
+		  " = ",			   /*                        */
+		  (val==NULL?"*NULL*":(char*)val));/*	                     */
+  }						   /*                        */
 						   /*			     */
 #define SETQ(V,T) V=T; ReleaseSymbol(name);
 #define RscNumeric(SYM,S,V,I)		\
-  if( name==S ) { SETQ(V,atoi(val));	     ReleaseSymbol(val); return 0; }
+  if( name==S ) { SETQ(V,atoi((char*)val));  ReleaseSymbol(val); return 0; }
 #define RscString(SYM,S,V,I)		\
   if( name==S ) { ReleaseSymbol(V); V = val; ReleaseSymbol(name); return 0; }
 #define RscBoolean(SYM,S,V,I)		\
@@ -291,7 +329,7 @@ int set_rsc(name,val)				   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void rsc_print(s)				   /*			     */
-  register char *s;				   /*			     */
+  char *s;				   	   /*			     */
 { ErrPrintF("%s\n",s);				   /* print the string itself*/
 			   			   /* followed by a newline  */
 }						   /*------------------------*/
