@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: s_parse.c,v 2.20 1997/12/06 22:56:00 gerd Exp gerd $
+** $Id: s_parse.c,v 2.21 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 ******************************************************************************/
 
@@ -27,10 +28,10 @@
 #else
 #define _ARG(A) ()
 #endif
- char * s_parse _ARG((int type,char **sp,int errp));/* s_parse.c             */
- int sp_open _ARG((char * s));			   /* s_parse.c              */
+ Uchar * s_parse _ARG((int type,Uchar **sp,int errp));/* s_parse.c           */
+ int sp_open _ARG((Uchar * s));			   /* s_parse.c              */
  void sp_close _ARG((void));			   /* s_parse.c              */
- void sp_error _ARG((char *s,char *a,char *b));	   /* s_parse.c              */
+ void sp_error _ARG((Uchar *s,char *a,char *b));   /* s_parse.c              */
 
 /*****************************************************************************/
 /* External Programs							     */
@@ -39,7 +40,7 @@
 /*---------------------------------------------------------------------------*/
 
 #define Error(E,S,A,B)	\
-  if(E) error(ERR_ERROR|ERR_POINT,A,B,(char*)0,sp_line,(Uchar*)S,0,(char*)0)
+  if(E) error(ERR_ERROR|ERR_POINT,(Uchar*)A,(Uchar*)B,(Uchar*)0,sp_line,(Uchar*)S,0,(char*)0)
 
  static Uchar *sp_line = NULL;
 
@@ -55,8 +56,8 @@
 ** Returns:	|TRUE|
 **___________________________________________________			     */
 int sp_open(s)					   /*                        */
-  register char * s;				   /*                        */
-{ sp_line = (Uchar*)s;				   /*                        */
+  Uchar *s;				   	   /*                        */
+{ sp_line = s;				   	   /*                        */
   return TRUE;					   /*                        */
 }						   /*------------------------*/
 
@@ -124,15 +125,15 @@ int sp_open(s)					   /*                        */
 **		message should be created in case of an error.
 ** Returns:	A symbol containing the requested entity or |NULL|.
 **___________________________________________________			     */
-char * s_parse(type,sp,errp)			   /*                        */
-  int 		type;				   /*                        */
-  char		**sp;				   /*                        */
-  int		errp;				   /*                        */
-{ register char *s = *sp;			   /*                        */
-  char          c,				   /*                        */
-  		*cp;				   /*                        */
-  static char   *unexpected = "Unexpected ";	   /*                        */
-  static char   *expected   = " expected.";	   /*                        */
+Uchar * s_parse(type,sp,errp)			   /*                        */
+  int 		 type;				   /*                        */
+  Uchar		 **sp;				   /*                        */
+  int		 errp;				   /*                        */
+{ register Uchar *s = *sp;			   /*                        */
+  char           c,				   /*                        */
+  		 *cp;				   /*                        */
+  static char    *unexpected = "Unexpected ";	   /*                        */
+  static char    *expected   = " expected.";	   /*                        */
  						   /*                        */
   while( is_space(*s) ) s++;			   /*                        */
   *sp = s;					   /*                        */
@@ -253,7 +254,9 @@ char * s_parse(type,sp,errp)			   /*                        */
   *sp = s;					   /*                        */
   if (   type == StringParseSymbol )		   /*                        */
   { (void)lower(cp); }			   	   /*                        */
-  s   = symbol(cp);				   /*                        */
-  free(cp);					   /*                        */
-  return s;					   /*                        */
+ 						   /*                        */
+  { Uchar *sym = symbol(cp);			   /*                        */
+    free(cp);					   /*                        */
+    return sym;					   /*                        */
+  }						   /*                        */
 }						   /*------------------------*/
