@@ -1,14 +1,15 @@
 /******************************************************************************
-** $Id: tex_aux.c,v 1.16 1997/12/03 11:50:30 gerd Exp $
+** $Id: tex_aux.c,v 1.17 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 ** 
 ******************************************************************************/
 
@@ -73,7 +74,7 @@ void clear_aux()				   /*                        */
 ** Returns:	|cite_star|
 **___________________________________________________			     */
 int foreach_aux(fct)				   /*                        */
-  int (fct)_ARG((char*));			   /*                        */
+  int (fct)_ARG((Uchar*));			   /*                        */
 { int i;					   /*                        */
   for ( i=0; i<32; i++ )			   /*                        */
   { foreach_word(cite[i],fct); }		   /*                        */
@@ -277,7 +278,7 @@ int apply_aux(db)				   /*                        */
 	 RecordIsXREF(rec)   &&			   /*                        */
 	 !RecordIsDELETED(rec)	 		   /*                        */
        )					   /*                        */
-    { char   *key;				   /*                        */
+    { Uchar  *key;				   /*                        */
       int    count;				   /*                        */
       Record r = rec;				   /*                        */
  						   /*                        */
@@ -291,10 +292,10 @@ int apply_aux(db)				   /*                        */
         if ( key == NULL )			   /*                        */
 	{ count = 0; }				   /*                        */
 	else					   /*                        */
-	{ key = symbol(expand_rhs(key,		   /*                        */
-				  sym_empty,	   /*                        */
-				  sym_empty,	   /*                        */
-				  db));		   /*                        */
+	{ key = symbol(lower(expand_rhs(key,	   /*                        */
+					sym_empty, /*                        */
+					sym_empty, /*                        */
+					db)));     /*                        */
 	  if ( (r=db_find(db,key)) == RecordNULL ) /*                        */
 	  { ErrPrintF("*** BibTool: Crossref `%s' not found.\n",key);/*      */
 	    count = 0;			   	   /*                        */
