@@ -1,16 +1,16 @@
 /******************************************************************************
-** $Id: tex_read.c,v 2.20 1997/12/03 11:50:30 gerd Exp $
+** $Id: tex_read.c,v 2.21 1999/12/18 22:04:46 gerd Exp $
 **=============================================================================
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-1997 Gerd Neugebauer
+** (c) 1996-1999 Gerd Neugebauer
 ** 
-** Net: gerd@informatik.uni-koblenz.de
+** Net: gerd.neugebauer@sdm.de
+**      gerd.neugebauer@gmx.de
 **	    
-** 
 ** Purpose: Simulate the TeX reading mechanism.
 **
 ** Compilation: cc tex_read.c -o tex_read -DSTANDALONE
@@ -28,6 +28,7 @@
 
 #ifndef STANDALONE
 #include <bibtool/error.h>
+#include <bibtool/symbols.h>
 #else
 #define Err(X)		 (void)fputs(X,stderr)
 #define ERROR_EXIT(X)	 Err(X); exit(1)
@@ -40,11 +41,12 @@
 
 #define CARET_CARET	    /* enable ^^A type specification of unprintables */
 #undef	TRANSLATE_NEWLINE   /* Translate nl to space or \par */
+#undef  USE_EXTENDED_AS_LETTERS /* Set the catcode of characters >=160 */
 
  typedef struct tOKEN				   /*			     */
- { short int	to_char;			   /*			     */
-   char		*to_s;				   /*			     */
-   struct tOKEN *to_next;			   /*			     */
+ { short int	 to_char;			   /*			     */
+   Uchar	 *to_s;				   /*			     */
+   struct tOKEN  *to_next;			   /*			     */
  } *Token, SToken;				   /*			     */
 
 #define TokenChar(X) ((X)->to_char)
@@ -55,7 +57,7 @@
 
  typedef struct mACdEF				   /*			     */
  { short int	 md_arity;			   /*			     */
-   char		 *md_name;			   /*			     */
+   unsigned char *md_name;			   /*			     */
    Token	 md_token;			   /*			     */
    struct mACdEF *md_next;			   /*			     */
  } *MacDef, SMacDef;				   /*			     */
@@ -75,31 +77,31 @@
 #else
 #define _ARG(A) ()
 #endif
- int TeX_read _ARG((char * cp,char **sp));	   /* tex_read.c             */
+ int TeX_read _ARG((Uchar * cp,Uchar **sp));	   /* tex_read.c             */
  static MacDef find_macro _ARG((char *name,MacDef md));/* tex_read.c         */
- static MacDef new_macdef _ARG((char *name,int arity,Token tokens));/* tex_read.c*/
+ static MacDef new_macdef _ARG((Uchar *name,int arity,Token tokens));/* tex_read.c*/
  static Token TeX_get_token _ARG((int (*get_fct)_ARG((void))));/* tex_read.c */
- static Token new_token _ARG((int type,char *string));/* tex_read.c          */
+ static Token new_token _ARG((int type,Uchar *string));/* tex_read.c         */
  static Token token_list_copy _ARG((Token t,Token nt,Token *argp));/* tex_read.c*/
- static Token tokenize _ARG((char *s,int arity));  /* tex_read.c             */
- static char * tokens_to_string _ARG((Token t));   /* tex_read.c             */
+ static Token tokenize _ARG((unsigned char *s,int arity));/* tex_read.c      */
  static int TeX_fill_line _ARG((int (*get_fct)_ARG((void))));/* tex_read.c   */
  static int do_get _ARG((void));		   /* tex_read.c             */
  static int fill_token _ARG((Token*tp));	   /* tex_read.c             */
  static int get_EOF _ARG((void));		   /* tex_read.c             */
  static int get_file _ARG((void));		   /* tex_read.c             */
  static int get_string _ARG((void));		   /* tex_read.c             */
+ static unsigned char * tokens_to_string _ARG((Token t));/* tex_read.c       */
  static void free_1_token _ARG((Token t));	   /* tex_read.c             */
  static void free_macdef _ARG((MacDef mac));	   /* tex_read.c             */
  static void free_tokens _ARG((Token t));	   /* tex_read.c             */
  static void init_TeX _ARG((void));		   /* tex_read.c             */
- static void init_get _ARG((char * s));		   /* tex_read.c             */
- void TeX_active _ARG((int c,int arity,char * s)); /* tex_read.c             */
+ static void init_get _ARG((Uchar*s));   	   /* tex_read.c             */
+ void TeX_active _ARG((int c,int arity,Uchar * s));/* tex_read.c             */
  void TeX_close _ARG((void));			   /* tex_read.c             */
- void TeX_def _ARG((char *s));			   /* tex_read.c             */
- void TeX_define _ARG((char *name,int arity,char *body));/* tex_read.c       */
+ void TeX_def _ARG((Uchar *s));		   	   /* tex_read.c             */
+ void TeX_define _ARG((Uchar *name,int arity,Uchar *body));/* tex_read.c     */
  void TeX_open_file _ARG((FILE * file));	   /* tex_read.c             */
- void TeX_open_string _ARG((char * s));		   /* tex_read.c             */
+ void TeX_open_string _ARG((unsigned char * s));   /* tex_read.c             */
  void TeX_reset _ARG((void));			   /* tex_read.c             */
 
 #ifdef STANDALONE
@@ -176,6 +178,9 @@ static void init_TeX()				   /*			     */
   for ( i=0; i<256; ++i )			   /*			     */
   { active[i]  = MacDefNULL;			   /*			     */
     catcode[i] = (  isalpha(i)			   /*			     */
+#ifdef USE_EXTENDED_AS_LETTERS
+		    || i >= 160			   /*                        */
+#endif
 		  ? CATCODE_LETTER		   /*			     */
 		  : CATCODE_OTHER  );		   /*			     */
   }						   /*			     */
@@ -209,9 +214,9 @@ static void init_TeX()				   /*			     */
 ** Returns:	the new token
 **___________________________________________________			     */
 static Token new_token(type,string)		   /*			     */
-  int	   type;				   /*			     */
-  char	   *string;				   /*			     */
-{ Token	   new;					   /*			     */
+  int	type;				   	   /*			     */
+  Uchar	*string;			   	   /*			     */
+{ Token new;					   /*			     */
 						   /*			     */
   if ( token_free_list != TokenNULL )		   /*			     */
   { new = token_free_list;			   /*			     */
@@ -316,19 +321,20 @@ static void free_tokens(t)			   /*			     */
 **	t
 ** Returns:	
 **___________________________________________________			     */
-static char * tokens_to_string(t)		   /*			     */
-  Token		 t;				   /*			     */
-{ register Token t1;				   /*			     */
-  register char	 *s,				   /*			     */
-		 *sp;				   /*			     */
-  register int	 len;				   /*			     */
+static unsigned char * tokens_to_string(t)	   /*			     */
+  Token			 t;			   /*			     */
+{ register Token	 t1;			   /*			     */
+  register unsigned char *s,			   /*			     */
+			 *sp;			   /*			     */
+  register int		 len;			   /*			     */
 						   /*			     */
   for ( len=1,t1=t;				   /* Count the elements of  */
 	t1!=TokenNULL;				   /*  the token list.	     */
 	t1=NextToken(t1) )			   /*  (+1 for '\0')	     */
   { ++len; }					   /*			     */
 						   /*			     */
-  if ( (s=malloc(len*sizeof(char))) == NULL )	   /* Try to get             */
+  if ( (s=(unsigned char*)malloc(len*sizeof(unsigned char)))/*               */
+       == NULL )	   			   /* Try to get             */
   { OUT_OF_MEMORY("TeX string."); } 		   /*     new memory.	     */
 						   /*			     */
   for ( sp=s,t1=t;				   /* Transfer the characters*/
@@ -437,7 +443,7 @@ static Token TeX_get_token(get_fct)		   /*			     */
 	switch (tex_state)			   /*			     */
 	{ case StateN:				   /*			     */
 	    TokenChar(t) = CHAR_ESCAPE;		   /*			     */
-	    TokenSeq(t)	 = new_string("par");	   /*			     */
+	    TokenSeq(t)	 = new_Ustring("par");	   /*			     */
 	    return t;				   /*			     */
 	  case StateM:				   /*			     */
 	    TokenChar(t) = CHAR_SPACE;		   /*			     */
@@ -462,7 +468,7 @@ static Token TeX_get_token(get_fct)		   /*			     */
 	}					   /*			     */
 	else if ( TokenChar(tex_line) == CHAR_EOL )/*			     */
 	{ tex_state   = StateM;			   /*			     */
-	  TokenSeq(t) = new_string("");		   /*			     */
+	  TokenSeq(t) = new_Ustring("");	   /*			     */
 	  return t;				   /*			     */
 	}					   /*			     */
 	t2	      = tex_line;		   /*			     */
@@ -481,8 +487,8 @@ static Token TeX_get_token(get_fct)		   /*			     */
   }						   /*			     */
 }						   /*------------------------*/
 
- static char *g_s;
- static char *g_p;
+ static unsigned char *g_s;
+ static unsigned char *g_p;
 
 /*-----------------------------------------------------------------------------
 ** Function:	init_get()
@@ -494,7 +500,7 @@ static Token TeX_get_token(get_fct)		   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 static void init_get(s)				   /*			     */
-  register char * s;				   /*			     */
+  register unsigned char * s;			   /*			     */
 { g_p = g_s = s;				   /*			     */
 }						   /*------------------------*/
 
@@ -522,7 +528,7 @@ static int do_get()				   /*			     */
 ** Returns:	
 **___________________________________________________			     */
 static Token tokenize(s,arity)			   /*			     */
-  char		*s;				   /*			     */
+  unsigned char	*s;				   /*			     */
   int		arity;				   /*			     */
 { Token		t ,				   /*			     */
 		t_ret = TokenNULL,		   /*			     */
@@ -562,7 +568,7 @@ static Token tokenize(s,arity)			   /*			     */
 ** Returns:	
 **___________________________________________________			     */
 static MacDef new_macdef(name,arity,tokens)	   /*			     */
-  char	    *name;				   /*			     */
+  Uchar	    *name;				   /*			     */
   int	    arity;				   /*			     */
   Token	    tokens;				   /*			     */
 { MacDef    new;				   /*			     */
@@ -577,13 +583,22 @@ static MacDef new_macdef(name,arity,tokens)	   /*			     */
   return new;					   /*			     */
 }						   /*------------------------*/
 
-static void free_macdef(mac)
-  MacDef mac;
-{
-  if ( mac == MacDefNULL ) return;
-  free(MacroToken(mac));
-  free(mac);
-}
+/*-----------------------------------------------------------------------------
+** Function:	free_macdef()
+** Purpose:	
+**		
+**		
+** Arguments:
+**	mac	
+** Returns:	nothing
+**___________________________________________________			     */
+static void free_macdef(mac)			   /*                        */
+  MacDef mac;					   /*                        */
+{						   /*                        */
+  if ( mac == MacDefNULL ) return;		   /*                        */
+  free(MacroToken(mac));			   /*                        */
+  free(mac);					   /*                        */
+}						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
 ** Function:	find_macro()
@@ -618,9 +633,9 @@ static MacDef find_macro(name,md)		   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void TeX_define(name,arity,body)		   /*			     */
-  char		  *name;			   /*			     */
+  Uchar		  *name;			   /*			     */
   int		  arity;			   /*			     */
-  char		  *body;			   /*			     */
+  Uchar		  *body;			   /*			     */
 { register MacDef md;				   /*			     */
 						   /*			     */
   if ( 0 > arity || arity > 9 ) return;		   /*			     */
@@ -646,10 +661,10 @@ void TeX_define(name,arity,body)		   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void TeX_def(s)					   /*			     */
-  register char *s;				   /*			     */
-{ char		*name,				   /*			     */
-		*ep   = NULL;			   /*			     */
-  char		c;				   /*			     */
+  Uchar *s;			   		   /*			     */
+{ Uchar	*name,				   	   /*			     */
+	*ep   = NULL;			   	   /*			     */
+  char	c;				   	   /*			     */
   int		arity = 0;			   /*			     */
 						   /*			     */
   while( isspace(*s) ) s++;			   /*                        */
@@ -665,13 +680,13 @@ void TeX_def(s)					   /*			     */
   if ( *name == '\\' )				   /*                        */
   {  						   /*                        */
     c = *ep; *ep = '\0';			   /*			     */
-    TeX_define(new_string(name+1),		   /*			     */
+    TeX_define(new_Ustring(name+1),	   	   /*			     */
 	       arity,				   /*			     */
-	       new_string(s+1));		   /*			     */
+	       new_Ustring(s+1));		   /*			     */
     *ep = c;					   /*			     */
   }						   /*                        */
   else						   /*                        */
-  { TeX_active(*name,arity,new_string(s+1));	   /*                        */
+  { TeX_active((unsigned int)*name,arity,new_Ustring(s+1));/*                */
   }						   /*                        */
 }						   /*------------------------*/
 
@@ -686,14 +701,14 @@ void TeX_def(s)					   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void TeX_active(c,arity,s)			   /*			     */
-  int  c;					   /*			     */
-  int  arity;					   /*			     */
-  char * s;					   /*			     */
+  int   c;					   /*			     */
+  int   arity;					   /*			     */
+  Uchar *s;				   	   /*			     */
 {						   /*			     */
   EnsureInit;					   /*			     */
-  if ( active[c] ) free_macdef(active[c]);
+  if ( active[c] ) free_macdef(active[c]);	   /*                        */
  						   /*                        */
-  active[c] = new_macdef((char*)NULL,		   /*                        */
+  active[c] = new_macdef((Uchar*)NULL,		   /*                        */
 			 arity,			   /*                        */
 			 tokenize(s,arity));	   /*	                     */
 }						   /*------------------------*/
@@ -735,8 +750,8 @@ static int get_EOF()				   /*			     */
 }						   /*------------------------*/
 
  static FILE *src_file;				   /*			     */
- static char *src_string;			   /*			     */
- static char *src_ptr;				   /*			     */
+ static unsigned char *src_string;		   /*			     */
+ static unsigned char *src_ptr;			   /*			     */
  static int  (*src_get)() = get_EOF;		   /*			     */
 
 /*-----------------------------------------------------------------------------
@@ -787,7 +802,7 @@ void TeX_open_file(file)			   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void TeX_open_string(s)				   /*			     */
-  char * s;					   /*			     */
+  unsigned char * s;				   /*			     */
 { src_ptr = src_string = s;			   /*			     */
   src_get = get_string;				   /*			     */
   tex_clear;					   /*			     */
@@ -802,7 +817,7 @@ void TeX_open_string(s)				   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void TeX_close()				   /*			     */
-{ char c, *s;					   /*			     */
+{ Uchar c, *s;				   	   /*			     */
   src_get = get_EOF;				   /*			     */
   while ( TeX_read(&c,&s) ) ;			   /*			     */
 }						   /*------------------------*/
@@ -816,10 +831,11 @@ void TeX_close()				   /*			     */
 ** Returns:	
 **___________________________________________________			     */
 static int fill_token(tp)			   /*			     */
-  register Token*tp;				   /*			     */
-{ if (	 *tp == TokenNULL			   /*			     */
+  register Token *tp;				   /*			     */
+{						   /*                        */
+  if (	 *tp == TokenNULL			   /*			     */
       && (*tp=TeX_get_token(src_get)) == TokenNULL )/*			     */
-  { return 0; }				   /*			     */
+  { return 0; }				   	   /*			     */
   return 1;					   /*			     */
 }						   /*------------------------*/
 
@@ -837,8 +853,8 @@ static int fill_token(tp)			   /*			     */
 ** Returns:	|FALSE| iff everything went right.
 **___________________________________________________			     */
 int TeX_read(cp,sp)				   /*			     */
-  char		* cp;				   /*			     */
-  char		**sp;				   /*			     */
+  Uchar		* cp;				   /*			     */
+  Uchar		**sp;				   /*			     */
 { static Token	t     = TokenNULL;		   /*			     */
   static Token	old_t = TokenNULL;		   /*			     */
   static Token	arg[10];			   /*			     */
@@ -850,64 +866,66 @@ int TeX_read(cp,sp)				   /*			     */
   if ( old_t != TokenNULL ) free_1_token(old_t);   /*			     */
 						   /*			     */
   while ( fill_token(&t) )			   /* One token left or	     */
-  {						   /* I got a new one	     */
-    if (   (mac=active[TokenChar(t)]) == MacDefNULL/*			     */
-	&& ( TokenChar(t) != CHAR_ESCAPE	   /* Is it a character?     */
-	   || (mac=find_macro(TokenSeq(t),macro))  /*  or an undefined	     */
-	      == MacDefNULL )  )		   /*  macro?		     */
+  {						   /*  I got a new one.	     */
+    if ( (mac=active[TokenChar(t)]) != MacDefNULL )/* Is it active?          */
+    { UnlinkAndFreeToken(t,t2);			   /* Delete active token    */
+    }						   /*                        */
+    else if ( TokenChar(t) == CHAR_ESCAPE &&	   /*                        */
+	      (mac=find_macro(TokenSeq(t),macro))  /*  or an undefined	     */
+	      != MacDefNULL )			   /*                        */
+    { UnlinkAndFreeToken(t,t2);			   /* Delete macro name token*/
+      while ( fill_token(&t)			   /* While there are more   */
+	     && TokenChar(t) == CHAR_SPACE)	   /*  tokens and == ' '     */
+      { UnlinkAndFreeToken(t,t2); }		   /* unlink space	     */
+    }						   /*                        */
+    else					   /*                        */
     { *cp = TokenChar(t);			   /*			     */
       *sp = TokenSeq(t);			   /*			     */
       UnlinkToken(t,old_t);			   /*			     */
       return 1;					   /*			     */
     }						   /*			     */
-    else					   /*			     */
-    { UnlinkAndFreeToken(t,t2);			   /* delete macro token     */
 						   /*			     */
-      while ( fill_token(&t)			   /* While there are more   */
-	     && TokenChar(t) == CHAR_SPACE)	   /*  tokens and == ' '     */
-      { UnlinkAndFreeToken(t,t2); }		   /* unlink space	     */
 						   /*			     */
-      for ( i=1; i<=MacroArity(mac); ++i )	   /* Fill the argument	     */
-      {						   /*  vector		     */
-	if ( !fill_token(&t) )			   /*			     */
-	{ arg[i] = TokenNULL;			   /* Not enough tokens	     */
-	  Err("*** Unexpected EOF\n");		   /*			     */
-	}					   /*			     */
-	else if ( TokenChar(t) == CHAR_BEG_GROUP ) /* If there is a group    */
-	{ tp = t;				   /*			     */
-	  d  = 0;				   /*			     */
-	  while ( fill_token(&NextToken(tp))  &&   /* While there are more   */
-		 (TokenChar(NextToken(tp))	   /*  tokens                */
-		  != CHAR_END_GROUP  		   /*  and                   */
+    for ( i=1; i<=MacroArity(mac); ++i )	   /* Fill the argument	     */
+    {						   /*  vector		     */
+      if ( !fill_token(&t) )			   /*			     */
+      { arg[i] = TokenNULL;			   /* Not enough tokens	     */
+        Err("*** Unexpected EOF\n");		   /*			     */
+      }					   	   /*			     */
+      else if ( TokenChar(t) == CHAR_BEG_GROUP )   /* If there is a group    */
+      { tp = t;				   	   /*			     */
+        d  = 0;				   	   /*			     */
+	while ( fill_token(&NextToken(tp))  &&     /* While there are more   */
+		(TokenChar(NextToken(tp))	   /*  tokens                */
+		 != CHAR_END_GROUP  		   /*  and                   */
 		 || d != 0 ) )			   /*  no matching '}'	     */
-	  { tp = NextToken(tp);			   /*  walk ahead and	     */
-	    switch ( TokenChar(tp) )		   /*  count braces	     */
-	    { case CHAR_BEG_GROUP: ++d; break;	   /*			     */
-	      case CHAR_END_GROUP: d--; break;	   /*			     */
-	    }					   /*			     */
+	{ tp = NextToken(tp);			   /*  walk ahead and	     */
+	  switch ( TokenChar(tp) )		   /*  count braces	     */
+	  { case CHAR_BEG_GROUP: ++d; break;	   /*			     */
+	    case CHAR_END_GROUP: d--; break;	   /*			     */
 	  }					   /*			     */
-	  t2 = NextToken(NextToken(tp));	   /*			     */
-	  free_1_token(NextToken(tp));		   /* remove final '}'	     */
-	  NextToken(tp) = TokenNULL;		   /*			     */
-	  tp = t; t = t2;			   /*			     */
-	  UnlinkAndFreeToken(tp,t2);		   /* remove initial '{'     */
-	  arg[i] = tp;				   /*			     */
 	}					   /*			     */
-	else					   /*			     */
-	{ arg[i] = t;				   /* Move one token	     */
-	  t = NextToken(t);			   /*  to argument vector    */
-	  NextToken(arg[i]) = TokenNULL;	   /*			     */
-	}					   /*			     */
-      }						   /*			     */
+	t2 = NextToken(NextToken(tp));	   	   /*			     */
+	free_1_token(NextToken(tp));		   /* remove final '}'	     */
+	NextToken(tp) = TokenNULL;		   /*			     */
+	tp = t; t = t2;			   	   /*			     */
+	UnlinkAndFreeToken(tp,t2);		   /* remove initial '{'     */
+	arg[i] = tp;				   /*			     */
+      }					   	   /*			     */
+      else					   /*			     */
+      { arg[i] = t;				   /* Move one token	     */
+        t = NextToken(t);			   /*  to argument vector    */
+	NextToken(arg[i]) = TokenNULL;	   	   /*			     */
+      }					   	   /*			     */
+    }						   /*			     */
 						   /*			     */
-      t = token_list_copy(MacroToken(mac),t,arg);  /*			     */
+    t = token_list_copy(MacroToken(mac),t,arg);    /*			     */
 						   /*			     */
-      for ( i=1; i<MacroArity(mac); ++i )	   /* free the arg vector    */
-      {	if ( arg[i] != TokenNULL )		   /*			     */
-	{ free_tokens(arg[i]);			   /*			     */
-	  arg[i] = TokenNULL;			   /*			     */
-	}					   /*			     */
-      }						   /*			     */
+    for ( i=1; i<MacroArity(mac); ++i )	   	   /* free the arg vector    */
+    { if ( arg[i] != TokenNULL )		   /*			     */
+      { free_tokens(arg[i]);			   /*			     */
+        arg[i] = TokenNULL;			   /*			     */
+      }					   	   /*			     */
     }						   /*			     */
   }						   /*			     */
   old_t = TokenNULL;				   /*			     */
